@@ -1,6 +1,6 @@
 /* ops.h - Internal operation support.
    Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003, 2004 g10 Code GmbH
+   Copyright (C) 2001, 2002, 2003, 2004, 2005 g10 Code GmbH
  
    This file is part of GPGME.
  
@@ -54,6 +54,11 @@ gpgme_error_t _gpgme_op_reset (gpgme_ctx_t ctx, int synchronous);
    KEY.  */
 gpgme_error_t _gpgme_parse_inv_recp (char *args, gpgme_invalid_key_t *key);
 
+/* Parse the PLAINTEXT status line in ARGS and return the result in
+   FILENAMEP.  */
+gpgme_error_t _gpgme_parse_plaintext (char *args, char **filenamep);
+
+
 
 /* From verify.c.  */
 gpgme_error_t _gpgme_op_verify_init_result (gpgme_ctx_t ctx);
@@ -98,9 +103,6 @@ gpgme_error_t _gpgme_passphrase_status_handler (void *priv,
 						gpgme_status_code_t code,
 						char *args);
 gpgme_error_t _gpgme_passphrase_command_handler (void *opaque,
-						 gpgme_status_code_t code,
-						 const char *key, int fd);
-gpgme_error_t _gpgme_passphrase_command_handler_internal (void *opaque,
 						 gpgme_status_code_t code,
 						 const char *key, int fd,
 						 int *processed);
@@ -136,9 +138,26 @@ void _gpgme_op_trustlist_event_cb (void *data, gpgme_event_io_t type,
 				   void *type_data);
 
 
-/*-- version.c --*/
-const char *_gpgme_compare_versions (const char *my_version,
-				     const char *req_version);
+/* From version.c.  */
+
+/* Return true if MY_VERSION is at least REQ_VERSION, and false
+   otherwise.  */
+int _gpgme_compare_versions (const char *my_version,
+			     const char *req_version);
 char *_gpgme_get_program_version (const char *const path);
 
+
+/* From sig-notation.c.  */
+
+/* Create a new, empty signature notation data object.  */
+gpgme_error_t _gpgme_sig_notation_create (gpgme_sig_notation_t *notationp,
+					  const char *name, int name_len,
+					  const char *value, int value_len,
+					  gpgme_sig_notation_flags_t flags);
+
+/* Free the signature notation object and all associated resources.
+   The object must already be removed from any linked list as the next
+   pointer is ignored.  */
+void _gpgme_sig_notation_free (gpgme_sig_notation_t notation);
+
 #endif /* OPS_H */
