@@ -1,22 +1,23 @@
 /* verify.c - Signature verification.
    Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003, 2004 g10 Code GmbH
+   Copyright (C) 2001, 2002, 2003, 2004, 2005 g10 Code GmbH
 
    This file is part of GPGME.
  
    GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- 
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
    GPGME is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
- 
-   You should have received a copy of the GNU General Public License
-   along with GPGME; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -57,10 +58,7 @@ release_op_data (void *hook)
 	{
 	  gpgme_sig_notation_t next_nota = notation->next;
 
-	  if (notation->name)
-	    free (notation->name);
-	  if (notation->value)
-	    free (notation->value);
+	  _gpgme_sig_notation_free (notation);
 	  notation = next_nota;
 	}
 
@@ -69,6 +67,9 @@ release_op_data (void *hook)
       free (sig);
       sig = next;
     }
+
+  if (opd->result.file_name)
+    free (opd->result.file_name);
 }
 
 
@@ -93,7 +94,8 @@ static void
 calc_sig_summary (gpgme_signature_t sig)
 {
   unsigned long sum = 0;
-
+  
+  /* Calculate the red/green flag.  */
   if (sig->validity == GPGME_VALIDITY_FULL
       || sig->validity == GPGME_VALIDITY_ULTIMATE)
     {
@@ -112,11 +114,6 @@ calc_sig_summary (gpgme_signature_t sig)
   else if (gpg_err_code (sig->status) == GPG_ERR_BAD_SIGNATURE)
     sum |= GPGME_SIGSUM_RED;
 
-  if (sig->validity == GPGME_VALIDITY_UNKNOWN)
-    {
-      if (gpg_err_code (sig->validity_reason) == GPG_ERR_CRL_TOO_OLD)
-	sum |= GPGME_SIGSUM_CRL_TOO_OLD;
-    }
 
   /* FIXME: handle the case when key and message are expired. */
   switch (gpg_err_code (sig->status))
@@ -142,6 +139,23 @@ calc_sig_summary (gpgme_signature_t sig)
       break;
     }
   
+  /* Now look at the certain reason codes.  */
+  switch (gpg_err_code (sig->validity_reason))
+    {
+    case GPG_ERR_CRL_TOO_OLD:
+      if (sig->validity == GPGME_VALIDITY_UNKNOWN)
+        sum |= GPGME_SIGSUM_CRL_TOO_OLD;
+      break;
+        
+    case GPG_ERR_CERT_REVOKED:
+      sum |= GPGME_SIGSUM_KEY_REVOKED;
+      break;
+
+    default:
+      break;
+    }
+
+  /* Check other flags. */
   if (sig->wrong_key_usage)
     sum |= GPGME_SIGSUM_BAD_POLICY;
   
@@ -189,6 +203,7 @@ parse_new_sig (op_data_t opd, gpgme_status_code_t code, char *args)
 {
   gpgme_signature_t sig;
   char *end = strchr (args, ' ');
+  char *tail;
 
   if (end)
     {
@@ -234,39 +249,70 @@ parse_new_sig (op_data_t opd, gpgme_status_code_t code, char *args)
       break;
 
     case GPGME_STATUS_ERRSIG:
-      if (end)
+      /* Parse the pubkey algo.  */
+      if (!end)
+	goto parse_err_sig_fail;
+      errno = 0;
+      sig->pubkey_algo = strtol (end, &tail, 0);
+      if (errno || end == tail || *tail != ' ')
+	goto parse_err_sig_fail;
+      end = tail;
+      while (*end == ' ')
+	end++;
+     
+      /* Parse the hash algo.  */
+      if (!*end)
+	goto parse_err_sig_fail;
+      errno = 0;
+      sig->hash_algo = strtol (end, &tail, 0);
+      if (errno || end == tail || *tail != ' ')
+	goto parse_err_sig_fail;
+      end = tail;
+      while (*end == ' ')
+	end++;
+
+      /* Skip the sig class.  */
+      end = strchr (end, ' ');
+      if (!end)
+	goto parse_err_sig_fail;
+      while (*end == ' ')
+	end++;
+
+      /* Parse the timestamp.  */
+      sig->timestamp = _gpgme_parse_timestamp (end, &tail);
+      if (sig->timestamp == -1 || end == tail || (*tail && *tail != ' '))
+	return gpg_error (GPG_ERR_INV_ENGINE);
+      end = tail;
+      while (*end == ' ')
+	end++;
+      
+      /* Parse the return code.  */
+      if (end[0] && (!end[1] || end[1] == ' '))
 	{
-	  int i = 0;
-	  /* The return code is the 6th argument, if it is 9, the
-	     problem is a missing key.  */
-	  while (end && i < 4)
+	  switch (end[0])
 	    {
-	      end = strchr (end, ' ');
-	      if (end)
-		end++;
-	      i++;
-	    }
-	  if (end && end[0] && (!end[1] || end[1] == ' '))
-	    {
-	      switch (end[0])
-		{
-		case '4':
-		  sig->status = gpg_error (GPG_ERR_UNSUPPORTED_ALGORITHM);
-		  break;
-		  
-		case '9':
-		  sig->status = gpg_error (GPG_ERR_NO_PUBKEY);
-		  break;
-		  
-		default:
-		  sig->status = gpg_error (GPG_ERR_GENERAL);
-		}
+	    case '4':
+	      sig->status = gpg_error (GPG_ERR_UNSUPPORTED_ALGORITHM);
+	      break;
+	      
+	    case '9':
+	      sig->status = gpg_error (GPG_ERR_NO_PUBKEY);
+	      break;
+	      
+	    default:
+	      sig->status = gpg_error (GPG_ERR_GENERAL);
 	    }
 	}
       else
-	sig->status = gpg_error (GPG_ERR_GENERAL);
-      break;
+	goto parse_err_sig_fail;
 
+      goto parse_err_sig_ok;
+      
+    parse_err_sig_fail:
+      sig->status = gpg_error (GPG_ERR_GENERAL);
+    parse_err_sig_ok:
+      break;
+      
     default:
       return gpg_error (GPG_ERR_GENERAL);
     }
@@ -285,7 +331,6 @@ static gpgme_error_t
 parse_valid_sig (gpgme_signature_t sig, char *args)
 {
   char *end = strchr (args, ' ');
-
   if (end)
     {
       *end = '\0';
@@ -302,6 +347,7 @@ parse_valid_sig (gpgme_signature_t sig, char *args)
   if (!sig->fpr)
     return gpg_error_from_errno (errno);
 
+  /* Skip the creation date.  */
   end = strchr (end, ' ');
   if (end)
     {
@@ -315,6 +361,43 @@ parse_valid_sig (gpgme_signature_t sig, char *args)
       sig->exp_timestamp = _gpgme_parse_timestamp (end, &tail);
       if (sig->exp_timestamp == -1 || end == tail || (*tail && *tail != ' '))
 	return gpg_error (GPG_ERR_INV_ENGINE);
+      end = tail;
+
+      while (*end == ' ')
+	end++;
+      /* Skip the signature version.  */
+      end = strchr (end, ' ');
+      if (end)
+	{
+	  while (*end == ' ')
+	    end++;
+
+	  /* Skip the reserved field.  */
+	  end = strchr (end, ' ');
+	  if (end)
+	    {
+	      /* Parse the pubkey algo.  */
+	      errno = 0;
+	      sig->pubkey_algo = strtol (end, &tail, 0);
+	      if (errno || end == tail || *tail != ' ')
+		return gpg_error (GPG_ERR_INV_ENGINE);
+	      end = tail;
+
+	      while (*end == ' ')
+		end++;
+
+	      if (*end)
+		{
+		  /* Parse the hash algo.  */
+
+		  errno = 0;
+		  sig->hash_algo = strtol (end, &tail, 0);
+		  if (errno || end == tail || *tail != ' ')
+		    return gpg_error (GPG_ERR_INV_ENGINE);
+		  end = tail;
+		}
+	    }
+	}
     }
   return 0;
 }
@@ -345,51 +428,39 @@ parse_notation (gpgme_signature_t sig, gpgme_status_code_t code, char *args)
 	   previous one.  The crypto backend misbehaves.  */
 	return gpg_error (GPG_ERR_INV_ENGINE);
 
-      notation = malloc (sizeof (*sig));
-      if (!notation)
-	return gpg_error_from_errno (errno);
-      notation->next = NULL;
+      err = _gpgme_sig_notation_create (&notation, NULL, 0, NULL, 0, 0);
+      if (err)
+	return err;
 
       if (code == GPGME_STATUS_NOTATION_NAME)
 	{
-	  int len = strlen (args) + 1;
-
-	  notation->name = malloc (len);
-	  if (!notation->name)
-	    {
-	      int saved_errno = errno;
-	      free (notation);
-	      return gpg_error_from_errno (saved_errno);
-	    }
-	  err = _gpgme_decode_percent_string (args, &notation->name, len);
+	  err = _gpgme_decode_percent_string (args, &notation->name, 0);
 	  if (err)
 	    {
-	      free (notation->name);
-	      free (notation);
+	      _gpgme_sig_notation_free (notation);
 	      return err;
 	    }
 
-	  notation->value = NULL;
+	  notation->name_len = strlen (notation->name);
+
+	  /* FIXME: For now we fake the human-readable flag.  The
+	     critical flag can not be reported as it is not
+	     provided.  */
+	  notation->flags = GPGME_SIG_NOTATION_HUMAN_READABLE;
+	  notation->human_readable = 1;
 	}
       else
 	{
-	  int len = strlen (args) + 1;
+	  /* This is a policy URL.  */
 
-	  notation->name = NULL;
-	  notation->value = malloc (len);
-	  if (!notation->value)
-	    {
-	      int saved_errno = errno;
-	      free (notation);
-	      return gpg_error_from_errno (saved_errno);
-	    }
-	  err = _gpgme_decode_percent_string (args, &notation->value, len);
+	  err = _gpgme_decode_percent_string (args, &notation->value, 0);
 	  if (err)
 	    {
-	      free (notation->value);
-	      free (notation);
+	      _gpgme_sig_notation_free (notation);
 	      return err;
 	    }
+
+	  notation->value_len = strlen (notation->value);
 	}
       *lastp = notation;
     }
@@ -429,6 +500,8 @@ parse_notation (gpgme_signature_t sig, gpgme_status_code_t code, char *args)
       err = _gpgme_decode_percent_string (args, &dest, len);
       if (err)
 	return err;
+
+      notation->value_len += strlen (dest);
     }
   else
     return gpg_error (GPG_ERR_INV_ENGINE);
@@ -612,6 +685,11 @@ _gpgme_verify_status_handler (void *priv, gpgme_status_code_t code, char *args)
       opd->only_newsig_seen = 0;
       break;
 
+    case GPGME_STATUS_PLAINTEXT:
+      err = _gpgme_parse_plaintext (args, &opd->result.file_name);
+      if (err)
+	return err;
+
     default:
       break;
     }
